//
// juma-qrss.c
//
//	QRSS/DFCW playing and message edit program for JUMA TX-136/500 by JL1VNQ / HARU
//
//
//	ver.0.60 27 Feb 2015
//	bugs fix(detect EEPROM initial data 0xFF, re-display when push Power off switch breaking)
//	code arrangemen

//	ver.0.50 17 Feb 2015
//	last saved string edit (no all clear)
//
//	ver.0.40 22 Jan 2015
//	"SPACE" for DFCW mode too long (5dots) fixed (3dots)
//
//	ver.0.30 12 Jan 2015
//	message edit, save, read from EEPROM implimented
//
//	ver.0.20 10 Jan 2015	
//	added DFCW mode, speed selection and playing display same as original CW beacon
//
//	ver.0.10 9 Jan 2015
//	first version (TX mode select menu, QRSS start and abort, CW port continuous on at tone_counter toword to zero)
//

#include "juma-tx500.h"
#include "DataEEPROM.h"
#include "lcd-tx500.h"
#include "tx500_eeprom.h"

//unsigned char qrss_char[33];
char qrss_char[33];
const char char_select[56] = {"ABCDEFGHIJKLMNOPQRSTUVWXYZ 0123456789 /?@:;+-=.,()!\"'_ "};

char qrss_mem_buf[33] = {0};					// message editing buffer 32 characters max

extern const char morse_char_table[68][8];
extern unsigned int qrss_playflag;				// 1:in play
extern unsigned int qrss_period[4];				// QRSS dot period table
extern int p_idx;								// period table index

unsigned int qrss_editflag = 0; 				// 1:in edit

unsigned int qrss_busy_counter = 0;
char lcd_qrss_buf[10] = {"         "};

extern unsigned long tw_shift;					// tword shift level
extern int tx_mode;

extern int str_edit;
extern int lcd_mode;
extern int sub_page1;
extern int edit_break;

extern int board_type;

extern void tx_on(void);
extern void tx_off(void);
extern int convert_ascii2morse_idx(unsigned char c);

extern unsigned int long_push;
extern void ms_delay(unsigned int);

extern void	clk_lcd(unsigned char);
extern void wait_lcd_rdy(void);
extern void set_cur_lcd(unsigned char);
extern void clear_lcd(void);
extern void lcd_putch(unsigned char);
extern void lcd_putst(register const char *);

extern int encoder_get(void);

extern void beep(int, unsigned int);

extern union
	{
	struct defval defval;
	unsigned int storage[sizeof(struct defval)/2];
	}eeprom;

extern union
	{
	struct calval calval;
	unsigned int ee[sizeof(struct calval)/2];
	}cal;

// in 1msec irq
void qrss_1s_counter(void){
	static unsigned int _1s_counter = 1000;
	if(qrss_busy_counter != 0){
		if(_1s_counter == 0){
			qrss_busy_counter--;
			_1s_counter = 1000;
		}
		else _1s_counter--;
	}
}

// in main routine

void save_qrss_string(void){
	unsigned int x, y, temp;
	for(x=0;x<32;x++){
		temp = (unsigned int)qrss_mem_buf[x];
		y = EraseEE(EEPAGE, (2*x+EEQRSS), WORD);		// EEPAGE = 0x7F, WORD = 1 (DataEEPROM.h)
		y = WriteEE(&temp, EEPAGE, (2*x+EEQRSS), WORD);	// EEPROM Address 8 high bits, address + physical EEPROM start 16 low bits
	}
}

void read_qrss_string(void){					// startup only
	unsigned int x,y,temp;

	for(x=0;x<32;x++){
		y = ReadEE(EEPAGE, (2*x+EEQRSS), &temp, WORD);
		qrss_char[x] = (char)temp;
		}
}

void erase_qrss_string(void){
	unsigned int x, y;
	for(x=0;x<32;x++){
		y = EraseEE(EEPAGE, (2*x+EEQRSS), WORD);		// EEPAGE = 0x7F, WORD = 1 (DataEEPROM.h)
	}
}

void qrss_lcd_curset(char idx){
	wait_lcd_rdy();
	LCD_RS = 0;
	if(idx > 15) set_cur_lcd(LINE2 + idx - 16);
	else set_cur_lcd(LINE1 + idx);
	LCD_RS = 1;
}

void cur_on(void){
		wait_lcd_rdy( );		// check lcd busy flag
		LCD_RS = 0;				// switch command mode		
		clk_lcd(0x0E);			// set cursor display on
		LCD_RS = 1;				// back to data mode
}

void cur_off(void){
		wait_lcd_rdy( );		// check lcd busy flag
		LCD_RS = 0;				// switch command mode		
		clk_lcd(0x0C);			// set cursor display off
		LCD_RS = 1;				// back to data mode
}

//
// QRSS/DFCW playing routine
//

void qrss_play(void){
	static unsigned int c_idx = 0;				// index of original characters
	static unsigned int m_idx;					// converted index
	
	static unsigned char _1char_outflag = 0;	// converted character playing flag
	static unsigned int s_idx = 0;				// signal index of converted character
	
	static unsigned char _1sig_outflag = 0;		// one signel playing flag
	static unsigned char tone_onflag = 0;		// tone sending flag
	
	static unsigned char morse_c;				// converted signal character(dot, dash, space, end mark)
	unsigned int i;	

	if(qrss_playflag != 0){
		if((_1char_outflag ==0) && ((qrss_char[c_idx] != 0) || (qrss_char[c_idx] != 0x0d))){
			m_idx = convert_ascii2morse_idx(qrss_char[c_idx]);
			_1char_outflag = 1;
			for(i=1;i<7;i++){
				lcd_qrss_buf[i] = lcd_qrss_buf[i+1];
			}
			lcd_qrss_buf[7] = qrss_char[c_idx];
		}
		else if((qrss_char[c_idx] == 0) || (qrss_char[c_idx] == 0x0d)){
			c_idx = 0;
			s_idx = 0;
			_1sig_outflag = 0;
			tone_onflag = 0;
			qrss_playflag = 0;
			for(i=0;i<8;i++){
				lcd_qrss_buf[i] = ' ';
			}
		}
		
		if((qrss_busy_counter == 0) && (_1sig_outflag == 0)){
			morse_c = morse_char_table[m_idx][s_idx];
			switch(morse_c){
				case 0: default:
					if(tx_mode == 1) qrss_busy_counter = qrss_period[p_idx] * 2;
					if(tx_mode == 2) qrss_busy_counter = qrss_period[p_idx];
					_1sig_outflag = 1;
					tone_onflag = 0;
					CW = 0;
					tw_shift = 0;
					lcd_qrss_buf[0] = ' ';
					break;
				case '.':
					if(tx_mode == 1) qrss_busy_counter = qrss_period[p_idx] * 2;
					if(tx_mode == 2)
						{
						if(morse_c == morse_char_table[m_idx][s_idx + 1])
							qrss_busy_counter = qrss_period[p_idx] + (qrss_period[p_idx] >> 2);  // gap = dot period / 4
						else
							qrss_busy_counter = qrss_period[p_idx];		// gap minimum
//						tw_shift = 27;									// -1Hz shift
						if(board_type == 1) tw_shift = 1400/qrss_period[p_idx];
						if(board_type == 2) tw_shift = 420/qrss_period[p_idx];
						}
					_1sig_outflag = 1;
					tone_onflag = 1;
					tx_on();
					lcd_qrss_buf[0] = morse_c;	
					break;
				case '-':
					if(tx_mode == 1) qrss_busy_counter = qrss_period[p_idx] * 3;
					if(tx_mode == 2)
						{
						if(morse_c == morse_char_table[m_idx][s_idx + 1])
							qrss_busy_counter = qrss_period[p_idx]+ (qrss_period[p_idx] >> 2);	// gap = dot period / 4
						else
							qrss_busy_counter = qrss_period[p_idx];		// gap minimum
						tw_shift = 0;									// 0Hz
						}
					_1sig_outflag = 1;
					tone_onflag = 1;
					tx_on();
					lcd_qrss_buf[0] = morse_c;
					break;
				case ' ':
					if(tx_mode == 1) qrss_busy_counter = qrss_period[p_idx] * 3;
					if(tx_mode == 2) qrss_busy_counter = qrss_period[p_idx];
					_1sig_outflag = 1;
					tone_onflag = 0;
					CW = 0;
					lcd_qrss_buf[0] = morse_c;
					break;
			}
		}
		else if(_1sig_outflag == 1){
			if(tx_mode == 1){
				if((qrss_busy_counter == qrss_period[p_idx]) && (tone_onflag != 0)){
						CW = 0;
						tone_onflag = 0;
						lcd_qrss_buf[0] = ' ';
				}
			}
			else if(tx_mode == 2){
				if(morse_c == morse_char_table[m_idx][s_idx + 1]){
					if((qrss_busy_counter == (qrss_period[p_idx] >> 2)) && (tone_onflag != 0)){
							CW = 0;
							tone_onflag = 0;
							lcd_qrss_buf[0] = ' ';
					}
				}
			}
			if(qrss_busy_counter == 0){
				_1sig_outflag = 0;
//				tx_off();
				if(morse_c == 0){
					_1char_outflag = 0;
					s_idx =0;
					c_idx++;
				}
				else{
					s_idx++;
				}
			}
		}
	}
	else{
			tx_off();
			tw_shift = 0;
			c_idx = 0;
			s_idx = 0;
			_1sig_outflag = 0;
			tone_onflag = 0;
			_1char_outflag = 0;
			for(i=0;i<8;i++){
				lcd_qrss_buf[i] = ' ';
			}
	}
}

//
// QRSS/DFCW message editing routine
//

void qrss_edit(void){
	
	static char buf_idx = 0;
	static char cs_idx = 0;
	unsigned char ind = 0;
	int i,temp;

	if(qrss_editflag == 0){

		qrss_editflag = 1;		// only one time

		clear_lcd();

		for(i=0;i<32;i++){
			qrss_mem_buf[i] = qrss_char[i];						// string copy from playing buffer
		}
		
		if((qrss_mem_buf[0] + 1) > 1){							// qrss_mem_buf[0] except 0xFF or 0x00
			while(qrss_mem_buf[ind] != 0){						// display saved string
				qrss_lcd_curset(ind);
				lcd_putch((unsigned char)qrss_char[ind]);
				ind++;
			}
			
			ind--;
			
			qrss_lcd_curset(ind);
			cur_on();

			for(i=0;i<56;i++){									// last char index search
			if(char_select[i] == qrss_mem_buf[ind - 1]) cs_idx = i;
			}
			buf_idx = ind;
		}
		else{
			qrss_mem_buf[0] = char_select[0];					// initial display "A"
			
			qrss_lcd_curset(0);
			lcd_putch('A');

			qrss_lcd_curset(0);
			cur_on();

			buf_idx = 0;
		}
	}

	if(edit_break){
		clear_lcd();
		
		for(i=0;i<buf_idx+1;i++){
			qrss_lcd_curset(i);
			lcd_putch(qrss_mem_buf[i]);
		}
		
		qrss_lcd_curset(buf_idx);
		cur_on();
		
		edit_break = 0;
	}

	temp = encoder_get();

	if(temp != 0){

		cur_off();

		cs_idx = cs_idx + temp;
		
		if(cs_idx < 0) cs_idx = 54;
		if(cs_idx >54) cs_idx = 0;
		
		qrss_mem_buf[(unsigned char)buf_idx] = char_select[(unsigned char)cs_idx];

		qrss_lcd_curset(buf_idx);
		lcd_putch(qrss_mem_buf[(unsigned char)buf_idx]);

		qrss_lcd_curset(buf_idx);								// cursor location reset
		cur_on();
	}

	if(DISP == 0)
		{
		long_push = LONG_PUSH;
		
		while(DISP == 0){
			if(long_push == 0){
				beep(TONE_C, cal.calval.beep_len*10);			// long beep for save
				break;
			}
		}
		
		if(long_push == 0){										// for save message
			save_qrss_string();
			for(i=0;i<32;i++){
				qrss_char[i] = qrss_mem_buf[i];					// string copy to playing buffer
			}
			
			clear_lcd();

			cur_off();

			set_cur_lcd(LINE1);
			lcd_putst("New QRSS String ");
			set_cur_lcd(LINE2);
			lcd_putst("Saved to EEPROM!");
			ms_delay(2000);

			buf_idx = 0;
			cs_idx = 0;
			str_edit = 0;
			lcd_mode = 0;
			sub_page1 = 0;
			qrss_editflag = 0;

			for(i=0;i<32;i++){									// clear buffer
				qrss_mem_buf[i] = 0;
			}													// to exit
		}
		else{													// for definition character
			if(buf_idx < 31){

				cur_off();			

				buf_idx++;
				qrss_mem_buf[(unsigned char)buf_idx] = char_select[(unsigned char)cs_idx];

				qrss_lcd_curset(buf_idx);
				lcd_putch(qrss_mem_buf[(unsigned char)buf_idx]);

				qrss_mem_buf[buf_idx + 1] = 0;

				qrss_lcd_curset(buf_idx);						// cursor location reset
				cur_on();
			}
		}
	}

	if(OPER == 0)
		{
		long_push = LONG_PUSH;
		
		while(OPER == 0){
			if(long_push == 0){
				beep(TONE_C, cal.calval.beep_len*10);			// long beep for cancel
				break;
			}
		}
		
		if(long_push == 0){										// for save message
			clear_lcd();

			cur_off();

			set_cur_lcd(LINE1);
			lcd_putst(" Save Cancelled ");
			set_cur_lcd(LINE2);
			lcd_putst("String no change");
			ms_delay(2000);

			buf_idx = 0;
			cs_idx = 0;
			str_edit = 0;
			lcd_mode = 0;
			sub_page1 = 0;
			qrss_editflag = 0;
			
			for(i=0;i<32;i++){									// clear buffer
				qrss_mem_buf[i] = 0;
			}
		}														// to exit
		else{													// for back space
			if(buf_idx > 0){

				cur_off();

				qrss_mem_buf[(unsigned char)buf_idx] = 0;

				qrss_lcd_curset(buf_idx);
				lcd_putch(' ');
				
				buf_idx--;

				qrss_lcd_curset(buf_idx);						// cursor location reset
				cur_on();

				for(i=0;i<56;i++){								// char index search
					if(char_select[i] == qrss_mem_buf[(unsigned char)buf_idx]) cs_idx = i;
				}
			}
		}
	}
}
